@@ -558,8 +558,6 @@ plugin_call (const struct plugin_list *pl,
       bool error = false;
       bool deferred = false;
       
-      mutex_lock_static (L_PLUGIN);
-
       setenv_del (es, "script_type");
       envp = make_env_array (es, false, &gc);
 
@@ -588,8 +586,6 @@ plugin_call (const struct plugin_list *pl,
       if (pr)
 	pr->n = i;
 
-      mutex_unlock_static (L_PLUGIN);
-
       gc_free (&gc);
 
       if (type == OPENVPN_PLUGIN_ENABLE_PF && success)
