@@ -168,7 +168,9 @@ status_flush (struct status_output *so)
 #if defined(HAVE_FTRUNCATE)
       {
 	const off_t off = lseek (so->fd, (off_t)0, SEEK_CUR);
-	ftruncate (so->fd, off);
+	if (ftruncate (so->fd, off) != 0) {
+	  msg (M_WARN, "Failed to truncate status file: %s", strerror(errno));
+	}
       }
 #elif defined(HAVE_CHSIZE)
       {
